
/*****************************************************************************
 * Preamble
 *****************************************************************************/

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#include <vlc_common.h>
#include <vlc_plugin.h>
#include <vlc_aout.h>

/*****************************************************************************
 * Local prototypes
 *****************************************************************************/
static int  Create    ( vlc_object_t * );
static void Destroy   ( vlc_object_t * );

static void DoWork    ( aout_instance_t *, aout_filter_t *, aout_buffer_t *,
                        aout_buffer_t * );
//Callback ( p_this, psz_name, oldval, val, p_config->p_callback_data )
//void SetValues(vlc_object_t *,char *, bool, bool, void *);						

/*****************************************************************************
 * Module descriptor
 *****************************************************************************/
vlc_module_begin();
    set_description( N_("Audio Panner") );
    set_shortname( N_("Panner") );
    set_category( CAT_AUDIO );
    set_subcategory( SUBCAT_AUDIO_AFILTER );
    set_capability( "audio filter", 50 );
    set_callbacks( Create, Destroy );
	//add_integer(name, value, p_callback, text, longtext, advc) 
	//add_bool("Turn On", false, SetValues, "Turn on Panner", "Turn On",true) ;
vlc_module_end();

/*****************************************************************************
 * Internal data structures
 *****************************************************************************/
struct aout_filter_sys_t
{
    int i_left;
    int i_right;
	int i_middle_left;
	int i_middle_right;
    int i_rear_left;
    int i_rear_right;
	int i_rear_center;
	int i_center;
	int i_lfe;
};
/*
		f-center(7)
f-left(0)			f-right(1)

m-left(2)			m-right(3)

r-left(4)			r-right(5)
		r-center(6)

	lfe(8)
*/

/* our internal channel order (WG-4 order) */
static const uint32_t pi_channels[] =
{ AOUT_CHAN_LEFT, AOUT_CHAN_RIGHT, AOUT_CHAN_MIDDLELEFT, AOUT_CHAN_MIDDLERIGHT,
  AOUT_CHAN_REARLEFT, AOUT_CHAN_REARRIGHT, AOUT_CHAN_REARCENTER,
  AOUT_CHAN_CENTER, AOUT_CHAN_LFE, 0 };

  //Callback ( p_this, psz_name, oldval, val, p_config->p_callback_data )
void SetValues(vlc_object_t * p_this,char *name, bool old_val, bool new_val, void * data)
{
msg_Info(p_this, "%d %d",old_val, new_val);

}

  
  
/*****************************************************************************
 * Create: allocate headphone downmixer
 *****************************************************************************/
static int Create( vlc_object_t *p_this )
{
    int i = 0;
    int i_offset = 0;
    aout_filter_t * p_filter = (aout_filter_t *)p_this;
	msg_Info(p_this, "Creating Panner");
	msg_Info(p_this, "ipc %x, ioc %x", p_filter->input.i_physical_channels, p_filter->input.i_original_channels); 
	msg_Info(p_this, "opc %x, ooc %x", p_filter->output.i_physical_channels, p_filter->output.i_original_channels); 
	//msg_Info(p_this, "if %x, of %x", p_filter->input.i_format, p_filter->output.i_format); 
	char * fcif = malloc(5*sizeof(char));
	char * fcof = malloc(5*sizeof(char));
	//memset ( fcif , 0 , 4*sizeof(char) );
	//memset ( fcof , 0 , 4*sizeof(char) );
	vlc_fourcc_to_char(p_filter->input.i_format, fcif);
	vlc_fourcc_to_char(p_filter->output.i_format, fcof);
	fcif[4] = '\0';
	fcof[4] = '\0';
	msg_Info(p_this, "fcif %s, fcof %s", fcif, fcof); 
	free(fcif);
	free(fcof);
	msg_Info(p_this, "ir %d, or %d", p_filter->input.i_rate, p_filter->output.i_rate); 
	msg_Info(p_this, "Nbi %d, Nbo %d",aout_FormatNbChannels( &p_filter->input ),aout_FormatNbChannels( &p_filter->output ));
    /* Validate audio filter format */
    
	
	// if ( p_filter->input.i_physical_channels != (AOUT_CHAN_LEFT|AOUT_CHAN_RIGHT))
	// {
		// msg_Info(p_this,"Break 1");
		// return -1;
	// }
    // if(! ( p_filter->input.i_original_channels & AOUT_CHAN_DOLBYSTEREO ))
	// {
		// return -1;
	// }
    // if(aout_FormatNbChannels( &p_filter->output ) <= 2)
	// {
		// return -1;
	// }
    // if( (p_filter->input.i_original_channels & ~AOUT_CHAN_DOLBYSTEREO )
          // != ( p_filter->output.i_original_channels & ~AOUT_CHAN_DOLBYSTEREO ) )
    // {
        // return -1;
    // }

    if ( p_filter->input.i_rate != p_filter->output.i_rate )
    {
		p_filter->output.i_rate = p_filter->input.i_rate;
		msg_Info(p_this,"Break 2");
        //return -1;
    }

    if ( p_filter->input.i_format != VLC_FOURCC('f','l','3','2'))
	{
		msg_Info(p_this,"Break 3");
        return -1;
    }
    if ( p_filter->output.i_format != VLC_FOURCC('f','l','3','2') )
    {
		msg_Info(p_this,"Break 4");
        return -1;
    }

	
	
    /* Allocate the memory needed to store the module's structure */
    p_filter->p_sys = malloc( sizeof(struct aout_filter_sys_t) );
    if ( p_filter->p_sys == NULL )
		return VLC_ENOMEM;
    p_filter->p_sys->i_left = -1;
    p_filter->p_sys->i_center = -1;
    p_filter->p_sys->i_right = -1;
    
	p_filter->p_sys->i_rear_left = -1;
    p_filter->p_sys->i_rear_center = -1;
    p_filter->p_sys->i_rear_right = -1;
	
	p_filter->p_sys->i_middle_left = -1;
	p_filter->p_sys->i_middle_right = -1;
	
	p_filter->p_sys->i_lfe = -1;

    while ( pi_channels[i] )
    {
        if ( p_filter->output.i_physical_channels & pi_channels[i] )
        {
            switch ( pi_channels[i] )
            {
                case AOUT_CHAN_LEFT:
                    p_filter->p_sys->i_left = i_offset;
                    break;
                case AOUT_CHAN_CENTER:
                    p_filter->p_sys->i_center = i_offset;
                    break;
                case AOUT_CHAN_RIGHT:
                    p_filter->p_sys->i_right = i_offset;
                    break;
                case AOUT_CHAN_REARLEFT:
                    p_filter->p_sys->i_rear_left = i_offset;
                    break;
                case AOUT_CHAN_REARCENTER:
                    p_filter->p_sys->i_rear_center = i_offset;
                    break;
                case AOUT_CHAN_REARRIGHT:
                    p_filter->p_sys->i_rear_right = i_offset;
                    break;
				case AOUT_CHAN_MIDDLELEFT:
                    p_filter->p_sys->i_middle_left = i_offset;
                    break;
				case AOUT_CHAN_MIDDLERIGHT:
                    p_filter->p_sys->i_middle_right = i_offset;
                    break;
				case AOUT_CHAN_LFE:
                    p_filter->p_sys->i_lfe = i_offset;
                    break;
            }
            ++i_offset;
        }
        ++i;
    }
	msg_Info(p_this, "i_left %d",p_filter->p_sys->i_left);
	msg_Info(p_this, "i_right %d",p_filter->p_sys->i_right);
	msg_Info(p_this, "i__middle_left %d",p_filter->p_sys->i_middle_left);
	msg_Info(p_this, "i_middle_right %d",p_filter->p_sys->i_middle_right);
	msg_Info(p_this, "i_rear_left %d",p_filter->p_sys->i_rear_left);
	msg_Info(p_this, "i_rear_right %d",p_filter->p_sys->i_rear_right);
	msg_Info(p_this, "i_rear_center %d",p_filter->p_sys->i_rear_center);
	msg_Info(p_this, "i_center %d",p_filter->p_sys->i_center);
	msg_Info(p_this, "i_lfe %d",p_filter->p_sys->i_lfe);
    p_filter->pf_do_work = DoWork;
    p_filter->b_in_place = 0;
	
	msg_Info(p_this,"Panner Working");
    return 0;
}

/*****************************************************************************
 * Destroy: deallocate resources associated with headphone downmixer
 *****************************************************************************/
static void Destroy( vlc_object_t *p_this )
{
    msg_Info(p_this, "Destroying Panner");
	aout_filter_t * p_filter = (aout_filter_t *)p_this;

    if ( p_filter->p_sys != NULL )
    {
        free ( p_filter->p_sys );
        p_filter->p_sys = NULL;
    }
}
/*****************************************************************************
 * DoWork: convert a buffer
 *****************************************************************************/
static void DoWork( aout_instance_t * p_aout, aout_filter_t * p_filter,
                    aout_buffer_t * p_in_buf, aout_buffer_t * p_out_buf )
{
	//msg_Info(p_aout,"Working");
    VLC_UNUSED(p_aout);
    float * p_in = (float*) p_in_buf->p_buffer;
    float * p_out = (float*) p_out_buf->p_buffer;
    size_t i_nb_samples = p_in_buf->i_nb_samples;
    size_t i_nb_channels = aout_FormatNbChannels( &p_filter->output );

    p_out_buf->i_nb_samples = i_nb_samples;
    p_out_buf->i_nb_bytes = sizeof(float) * i_nb_samples
                            * aout_FormatNbChannels( &p_filter->output );
    memset ( p_out , 0 , p_out_buf->i_nb_bytes );

    if (( p_filter->p_sys != NULL ))
    {
        struct aout_filter_sys_t * p_sys = p_filter->p_sys;
        size_t i;

		float f_left = 0;
		float f_right = 0;
		float f_middle_left = 0;
		float f_middle_right = 0;
		float f_rear_left = 0;
		float f_rear_right = 0;
		float f_rear_center = 0;
		float f_center = 0;
		float f_lfe = 0;
	
		
        for ( i = 0; i < i_nb_samples; ++i )
        {
            
			
			if(p_sys->i_left >= 0){
				f_left = p_in[ i * i_nb_channels + p_sys->i_left];
			}
			if(p_sys->i_right >= 0){
				f_right = p_in[ i * i_nb_channels + p_sys->i_right];
			}
			
			
			if(p_sys->i_middle_left >= 0){
				f_middle_left = p_in[ i * i_nb_channels + p_sys->i_middle_left];
			}
			if(p_sys->i_middle_right >= 0){
				f_middle_right = p_in[ i * i_nb_channels + p_sys->i_middle_right];
			}
			
			
			if(p_sys->i_rear_left >= 0){
				f_rear_left = p_in[ i * i_nb_channels + p_sys->i_rear_left];
			}
			if(p_sys->i_rear_right >= 0){
				f_rear_right = p_in[ i * i_nb_channels + p_sys->i_rear_right];
			}
			
			
			if(p_sys->i_rear_center >= 0){
				f_rear_center = p_in[ i * i_nb_channels + p_sys->i_rear_center];
			}
			if(p_sys->i_center >= 0){
				f_center = p_in[ i * i_nb_channels + p_sys->i_center];
			}
			
			
			if(p_sys->i_lfe >= 0){
				f_lfe = p_in[ i * i_nb_channels + p_sys->i_lfe];
			}
						
			
			if ( p_sys->i_left >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_left ] = f_left;
            }
            if ( p_sys->i_right >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_right ] = f_right;
            }
			
			
			if ( p_sys->i_middle_left >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_middle_left ] = f_middle_left;
            }
            if ( p_sys->i_right >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_middle_right ] = f_middle_right;
            }
            
			
			if ( p_sys->i_rear_left >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_rear_left ] = f_rear_left;
            }
            if ( p_sys->i_rear_right >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_rear_right ] = f_rear_right;
            }
			
			
			if ( p_sys->i_rear_center >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_rear_center ] = f_rear_center;
            }
			if ( p_sys->i_center >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_center ] = f_center;
            }
			
			if ( p_sys->i_lfe >= 0 )
            {
                p_out[ i * i_nb_channels + p_sys->i_lfe ] = f_lfe;
            }
        }
    }
	
}
